import 'dart:html';

import 'package:http/http.dart' as http;
import 'package:logger/logger.dart';

abstract class DooadexLogger {
  factory DooadexLogger(String message) = _DooadexLogger;

  factory DooadexLogger.exception(Exception exception, {int? methodCount}) = _ExceptionLogger;

  factory DooadexLogger.httpRequest({required http.Request httpRequest}) = _HttpRequestLogger;

  factory DooadexLogger.httpResponse({required http.Response httpResponse, bool headers}) = _HttpResponseLogger;

  factory DooadexLogger.error(Error error, {int? methodCount}) = _ErrorLogger;

  factory DooadexLogger.apiRequest(
      {required String method,
      required Headers headers,
      required Uri uri,
      Map? body,
      int? methodCount}) = _ApiRequestLogger;

  factory DooadexLogger.apiResponse() = _ApiResponseLogger;

  factory DooadexLogger.htmlRequest(
      {required String method,
      required Map<String, String> headers,
      required Uri uri,
      String? body,
      int? methodCount}) = _HtmlRequestLogger;

  factory DooadexLogger.htmlResponse(
      {Map<String, String>? headers,
      required int statusCode,
      required String body,
      int? methodCount}) = _HtmlResponseLogger;
}

class _DooadexLogger implements DooadexLogger {
  _DooadexLogger(String message, {int? methodCount}) {
    Logger(printer: PrettyPrinter(methodCount: methodCount ?? 0)).v(message);
  }
}

class _ExceptionLogger implements DooadexLogger {
  _ExceptionLogger(Exception exception, {int? methodCount}) {
    Logger(printer: PrettyPrinter(methodCount: methodCount ?? 15)).e(exception);
  }
}

class _ErrorLogger implements DooadexLogger {
  _ErrorLogger(Error error, {int? methodCount}) {
    Logger(printer: PrettyPrinter(methodCount: methodCount ?? 0)).e(error);
  }
}

class _HttpRequestLogger implements DooadexLogger {
  _HttpRequestLogger({required http.Request httpRequest}) {
    Logger(printer: PrettyPrinter(methodCount: 0)).d(
        "Http Request ${DateTime.now()}\nURI: ${httpRequest.url}\nMethod: ${httpRequest.method}\nheaders: ${httpRequest.headers}\nBody: ${httpRequest.body}");
  }
}

class _HttpResponseLogger implements DooadexLogger {
  _HttpResponseLogger({required http.Response httpResponse, bool headers = false}) {
    headers == false
        ? Logger(printer: PrettyPrinter(methodCount: 0))
            .d("Http Response ${DateTime.now()}\nStatus Code: ${httpResponse.statusCode}\nBody: ${httpResponse.body}")
        : Logger(printer: PrettyPrinter(methodCount: 0)).d(
            "Http Response ${DateTime.now()}\nStatus Code: ${httpResponse.statusCode}\nHeaders ${httpResponse.headers}\nBody: ${httpResponse.body}");
  }
}

class _ApiRequestLogger implements DooadexLogger {
  _ApiRequestLogger({required String method, required Headers headers, required Uri uri, Map? body, int? methodCount}) {
    Logger(printer: PrettyPrinter(methodCount: methodCount ?? 0))
        .d("Request ${DateTime.now()}\nURI: $uri\nMethod: $method\nheaders: $headers\nBody: $body");
  }
}

class _HtmlRequestLogger implements DooadexLogger {
  _HtmlRequestLogger(
      {required String method,
      required Map<String, String> headers,
      required Uri uri,
      String? body,
      int? methodCount}) {
    Logger(printer: PrettyPrinter(methodCount: methodCount ?? 0))
        .d("Request ${DateTime.now()}\nURI: $uri\nMethod: $method\nheaders: $headers\nBody: $body");
  }
}

class _ApiResponseLogger implements DooadexLogger {
  _ApiResponseLogger();
}

class _HtmlResponseLogger implements DooadexLogger {
  _HtmlResponseLogger({Map<String, String>? headers, required int statusCode, required String body, int? methodCount}) {
    headers == null
        ? Logger(printer: PrettyPrinter(methodCount: methodCount ?? 0))
            .d("${DateTime.now()}\nResponse\nStatus Code: $statusCode\nBody: $body")
        : Logger(printer: PrettyPrinter(methodCount: 0))
            .d("${DateTime.now()}\nResponse\nStatus Code: $statusCode\nHeaders $headers\nBody: $body");
  }
}

